unit SMIMEModule;

interface

uses
  System.SysUtils, System.Classes, stCustomProtoComp, stSecComp,
  StreamSec.DSI.KeyManager, StreamSec.DSI.PkixKeyManager, StreamSec.DSI.PkixCms,
  stSMIMEId10Broker, StreamSec.DSI.KeyStorageFile, StreamSec.DSI.PkixKeyIndex,
  StreamSec.DSI.PkixKeyFilter, IdBaseComponent, IdMessage, Vcl.ExtCtrls;

type
  TdmSMIME = class(TDataModule)
    stSMIMEIdBroker1: TstSMIMEIdBroker;
    stMPEnvelopedData1: TstMPEnvelopedData;
    stMPSignedData1: TstMPSignedData;
    stCertificateManager1: TstCertificateManager;
    KeyStorageFiles1: tKeyStorageFiles;
    KeyStorageFiles2: tKeyStorageFiles;
    KeyStorageFiles3: tKeyStorageFiles;
    KeyIndexSerialIssuerCMS1: tKeyIndexSerialIssuerCMS;
    KeyIndexSubjectPublicKeyIdentifier1: tKeyIndexSubjectPublicKeyIdentifier;
    KeyFilterChain1: tKeyFilterChain;
    KeyIndexTBSThumbprint1: tKeyIndexTBSThumbprint;
    KeyIndexSerialIssuer1: tKeyIndexSerialIssuer;
    KeyIndexSubjectClearTextName1: tKeyIndexSubjectClearTextName;
    Timer1: TTimer;
    KeyIndexSubjectClearTextName2: tKeyIndexSubjectClearTextName;
    KeyIndexSubjectPublicKeyIdentifier2: tKeyIndexSubjectPublicKeyIdentifier;
    KeyIndexPublicKey1: tKeyIndexPublicKey;
    IdMessage1: TIdMessage;
    procedure DataModuleDestroy(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure KeyStorageFiles1GetNewKeyFileName(Sender: TObject; const aKey:
        IInterface; var aFileName: string);
    procedure KeyStorageFiles1SuggestFileKeyFormat(Sender: TObject; const
        aFileName: string; aFormats: IInterfaceList);
    procedure Timer1Timer(Sender: TObject);
  private
    fActions: TStrings;
    function ExtractEmail(const aStr: string): string;
  public
    destructor Destroy; override;
    procedure ImportPFX(const aFilename: string);
    procedure ImportRoot(const aFilename: string);
    procedure ImportCer(const aFilename: string);
    function LoadEml(const aFilename: string): TIdMessage;
    procedure SaveEml(aMsg: TIdMessage; const aFilename: string);
    property EncodingActions: TStrings read fActions;
  end;

var
  dmSMIME: TdmSMIME;

implementation

uses
  Dialogs,
  stGC,
  stSHA1,
  stSHA256, stSHA512,
  stAes,
  stArc2,
  st3DES,
  stCamellia,
  stDrbgAesCtr,
  stRandom, stSecUtils,
  StreamSec.DSI.PkixCert,
  StreamSec.DSI.Cms,
  StreamSec.DSI.NetFormat,
  StreamSec.DSI.IFC,
  StreamSec.DSI.X509Format,
  stDERUtils,
  stDERCoder;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

procedure TdmSMIME.DataModuleDestroy(Sender: TObject);
begin
  stCertificateManager1.Enabled := False;
  fActions.Free;
  stGC.tGCManager.GetDefault.Sweep;
  stGC.tGCManager.GetDefault.Sweep;
end;

destructor TdmSMIME.Destroy;
begin
  inherited;
  stGC.tGCManager.GetDefault.Sweep;
  stGC.tGCManager.GetDefault.Sweep;
end;

function TdmSMIME.ExtractEmail(const aStr: string): string;
var
  lPos: Integer;
begin
  lPos := Pos('<',aStr);
  if lPos = 0 then
    Result := aStr
  else begin
    Result := Copy(aStr,lPos+1,MaxInt);
    lPos := Pos('>',Result);
    if lPos = 0 then
      raise Exception.Create('Error Message')
    else
      Result := Copy(Result,1,lPos-1);
  end;
end;

procedure TdmSMIME.DataModuleCreate(Sender: TObject);
begin
  fActions := TStringList.Create;
  if not DirectoryExists('.\Data') then
    stCertificateManager1.ForceDirectories;
  stCertificateManager1.Enabled := True;
  KeyStorageFiles1.Enabled := True;
  KeyStorageFiles2.Enabled := True;
  KeyStorageFiles3.Enabled := True;
//  KeyIndexSerialIssuerCMS1.Enabled := True;
//  KeyIndexSerialIssuer1.Enabled := True;
  KeyIndexSubjectPublicKeyIdentifier1.Enabled := True;
//  KeyIndexDigest1.Enabled := True;
end;

{ TDataModule1 }

procedure TdmSMIME.ImportCer(const aFilename: string);
var
  lCer: iCertificate;
begin
  lCer := tCertificate.Create(gcamFirstAssignment);
  lCer.GetStruct.LoadFromFile(aFilename,fmtDER);
  KeyStorageFiles3.SetKey(lCer);
end;

procedure TdmSMIME.ImportPFX(const aFilename: string);
var
  lPwd: string;
begin
  if InputQuery('Password for ' + aFilename,'Enter',lPwd) then begin
    stCertificateManager1.ImportFromPFXFile(aFilename,lPwd);
  end;
end;

procedure TdmSMIME.ImportRoot(const aFilename: string);
var
  lCer: iCertificate;
begin
  lCer := tCertificate.Create(gcamFirstAssignment);
  lCer.GetStruct.LoadFromFile(aFilename,fmtDER);
  stCertificateManager1.SetCACertificate(lCer);
end;

procedure TdmSMIME.KeyStorageFiles1GetNewKeyFileName(Sender: TObject; const
    aKey: IInterface; var aFileName: string);
var
  lID: OctetString;
begin
  SetLength(lID,8);
  lID := KeyIndexSubjectPublicKeyIdentifier1.GetIdentifier(aKey);
  if Supports(aKey,iCertificate) then
    aFileName := OSToHex(lID,True) + '.cer'
  else
    aFileName := OSToHex(lID,True) + '.xml'
end;

procedure TdmSMIME.KeyStorageFiles1SuggestFileKeyFormat(Sender: TObject; const
    aFileName: string; aFormats: IInterfaceList);
begin
  if SameText(ExtractFileExt(aFileName),'.cer') then
    aFormats.Add(KeyStorageFiles1.PublicKeyFormatIF.EnabledFormats[0])
  else
    aFormats.Add(KeyStorageFiles1.PrivateKeyFormatIF.EnabledFormats[0]);
end;

function TdmSMIME.LoadEml(const aFilename: string): TIdMessage;
var
  lSrc: TIdMessage;
begin
  try
    Result := TIdMessage.Create(nil);
    lSrc := TIdMessage.Create(nil);
    try
      lSrc.LoadFromFile(aFilename);
      fActions.Clear;
      if not stSMIMEIdBroker1.Decode(lSrc,Result,fActions) then
        raise Exception.Create('Error Message');
    finally
      lSrc.Free;
    end;
  except
    Result.Free;
    raise;
  end;
end;

procedure TdmSMIME.SaveEml(aMsg: TIdMessage; const aFilename: string);
var
  lDst: TIdMessage;
  lRes: Boolean;
begin
  lDst := TIdMessage.Create(nil);
  try
    if stCertificateManager1.GetCertificate('/E='+aMsg.Recipients.Items[0].Address) = nil then
      lRes := stSMIMEIdBroker1.Encode(MakeContentTypeArray([pSignedData]),aMsg,lDst)
    else
      lRes := stSMIMEIdBroker1.Encode(MakeContentTypeArray([pSignedData,pEnvelopedData]),aMsg,lDst);
    if not lRes then
      raise Exception.Create('Error Message');
    lDst.SaveToFile(aFilename);
  finally
    lDst.Free;
  end;
end;

procedure TdmSMIME.Timer1Timer(Sender: TObject);
begin
  stGC.tGCManager.GetDefault.Sweep;
end;

end.
